import random
import sys
import hashlib
import base64
def modular_exp(b, e, m):
    r = 1
    while e > 0:
        if e & 1:
            r = (r * b) % m
        e >>= 1
        b = (b ** 2) % m
    return r
def lista_primos(t):
    l = [2]
    i = 3
    while i < t:
        primo = 1
        for c in l:
            if i % c == 0:
                primo = 0
                break
        if primo:
            l.append(i)
        i += 2
    return l
primelist = lista_primos(5000)
def firstcheck(n):
    for p in primelist:
        if n%p==0:
            if n==p:
                return 1
            else:
                return 0
    return 1
def fermat(n,k=100):
    for i in range(0, k):
        a = random.randint(1,n-1)
        if modular_exp(a,n-1,n) !=1:
            return 0
    return 1
def binary(n):
    l = []
    while (n > 0):
        l.append(n % 2)
        n /= 2
    return l
    
def miller_rabin(a, n): #1 = Complex, 0 = Primo
    b = binary(n - 1)
    d = 1
    for i in xrange(len(b) - 1, -1, -1):
        x = d
        d = (d**2) % n
        if d == 1 and x != 1 and x != n -1:
            return 1
        if b[i] == 1:
            d = (d*a) % n
    if d != 1:
        return 1
    return 0
def check_prime(n, k=100):
    if (not firstcheck(n)):
        return 0
    if (not fermat(n,k)):
        return 0
    for iteration in range(0,k):
        i = random.randint(1, n-1)
        if miller_rabin(i, n):
            return 0
    return 1
def modular_mi_extended(a, b):
    x, last_x = 0, 1
    y, last_y = 1, 0
    while b:
        q = a // b
        a, b = b, a % b
        x, last_x = last_x - q*x, x
        y, last_y = last_y - q*y, y
    return (last_x, last_y, a)
def modular_mi(a, m):
    x, q, gcd = modular_mi_extended(a, m)
    if gcd == 1:
        return (x + m) % m
    else:
        return None
def generar_primo(bitn=512, k=100):
    primo = random.randint(2**(bitn-1), 2**bitn)
    primo |= 1
    while not check_prime(primo, k):
        primo += 2
    return primo
try:
    import psyco
    psyco.full()
except ImportError:
    pass
def generar_claves(bitn=512):
    e=65537
    p = generar_primo(bitn)
    q = generar_primo(bitn)
    n = p * q #DECLARACION DE LA CLAVE PUBLICA
    phi = (p - 1) * (q - 1)
    while (phi%e) == 0:
        e = generar_primo(17)
    d = modular_mi(e, phi) #DECLARACION DE LA CLAVE PRIVADA
    public = n
    private = d
    return base64.b64encode(str(public)), base64.b64encode(str(private))
def str_to_int(s):
    n = 0
    for c in s:
        n <<= 8
        n += ord(c)
    return n
def int_to_str(n):
    s = []
    while n > 0:
        s.insert(0, chr(n & 255))
        n >>= 8
    return ''.join(s)
def encriptar(s, public):
    public = int(base64.b64decode(public))
    n = str_to_int(s)
    c = modular_exp(n, 65537, public)
    return base64.b64encode(str(c))
def desencriptar(n, private, public):
    n = int(base64.b64decode(n))
    public = int(base64.b64decode(public))
    private = int(base64.b64decode(private))
    p = modular_exp(n, private, public)
    s = int_to_str(p)
    return s
def sign(msg, private, public):
    private = int(base64.b64decode(private))
    public = int(base64.b64decode(public))
    s = int(hashlib.sha1(msg).hexdigest(), 16)
    return base64.b64encode(str(modular_exp(s, private, public)))
def checksign(msg, sign, pub):
    sign = int(base64.b64decode(sign))
    pub = int(base64.b64decode(pub))
    s = modular_exp(sign,65537,pub)
    h = int(hashlib.sha1(msg).hexdigest(), 16)
    return s == h



from packages import pyDes as pydes
def encriptar(archivo, passwd):
    f = open(archivo, "rb+")
    d = f.read()
    f.close()
    k = pydes.des(passwd)
    d = k.encrypt(d, ' ')
    f = open(archivo, 'wb+')
    f.write(d)
    f.close()
    return 1
def desencriptar(archivo, passwd):
    f = open(archivo, "rb+")
    d = f.read()
    f.close()
    k = pydes.des(passwd)
    d = k.decrypt(d, ' ')
    f = open(archivo, 'wb+')
    f.write(d)
    f.close()
    return 1



import crypto
class config():
    def __init__(self, archivo, encryption):
        if(encryption):
            self.encryption = True
            self.archivo = archivo
            self.passwd = "ShR0hv71" #patron de encriptacion
        else:
            self.encryption = False
            self.archivo = archivo
        self.opciones = {}
    def create(self):
        f = open(self.archivo, "w")
        f.close()
    def load(self):
        archivo = self.archivo
        if(self.encryption):
            crypto.desencriptar(self.archivo, self.passwd)
            f = open(self.archivo, "r")
            self.txt = f.read()
            f.close()
            crypto.encriptar(self.archivo, self.passwd)
        else:
            f = open(self.archivo, "r")
            self.txt = f.read()
            f.close()
        #carga, <--new-!>hola<--set-!>bye
        self.txt = self.txt.split("<--new-!>")
        v1 = ""
        for i in self.txt:
            if i == "":
                continue
            v1 = i.split("<--set-!>")
            self.opciones[v1[0]] = v1[1]
    def get_value(self, variable):
        return self.opciones[variable]
    def set_value(self, variable, valor):
        self.opciones[variable] = valor
        return self.opciones[variable]
    def save(self):
        keys = self.opciones.keys()
        txt = ""; txt_1 = ""
        for i in range(len(self.opciones)):
            variable = keys[i]
            valor = self.opciones[keys[i]]
            txt_1 = "<--new-!>%s<--set-!>%s" % (variable, valor)
            txt = txt + txt_1
        f = open(self.archivo, "w")
        f.write(txt)
        f.close()
        if(self.encryption):
            crypto.encriptar(self.archivo, self.passwd)

        
         
            



import progressbar as pb
import bitcoin as btc
import cfg
import rsa_def
import cyfsql

class ProgressBar():
    def __init__(self, **kwargs):
        self.progressbarvar = pb.progressbar(**kwargs)
        return None
    def Update(self, message=None):
        self.progressbarvar.print_progress_bar(message)
    def Increment(self, incremento):
        self.progressbarvar.increment(incremento)
    def Reset(self):
        self.progressbarvar.progress = 0
    def Percentage(self):
        return self.progressbarvar.progress
    def PrintBar(self):
        return self.progressbarvar.return_barra()

class BitCoin():
    def __init__(self):
      self.bitcoin = btc.BitCoin()
      return None
    def GetAddress(self, seed):
      return self.bitcoin.GetAddress(seed)
    def GetPrivateKey(self, seed):
      return self.bitcoin.GetPrivKey(seed)
    def CheckBalance(self, address):
      return self.bitcoin.CheckBalance(address)
    def ConvertAmount(self, decimal):
      return self.bitcoin.TransformBTC(decimal)
    def AddressByPrivateKey(self, privatekey):
      return self.bitcoin.AddressByPrivateKey(privatekey)
    def RandomSeed(self):
      return self.bitcoin.RandomSeed()

class ConfigManager():
    def __init__(self, archivo, encryption=True):
        self.cfgm = cfg.config(archivo, encryption)
        return None
    def Load(self):
        return self.cfgm.load()
    def Save(self):
        return self.cfgm.save()
    def GetValue(self, variable):
        return self.cfgm.get_value(variable)
    def SetValue(self, variable, valor):
        return self.cfgm.set_value(variable, valor)
    def Create(self):
        self.cfgm.create()
    def Reload(self):
        return self.cfgm.load()

class RSA():
    def __init__(self):
        return None
    def GenerateKeys(self, bits=512): #Devuelve (clave_publica, clave_privada)
        return rsa_def.generar_claves(bits)
    def Crypt(self, mensaje, clave_publica):
        return rsa_def.encriptar(mensaje, clave_publica)
    def Decrypt(self, mensaje_encriptado, clave_privada, clave_publica):
        return rsa_def.desencriptar(mensaje_encriptado, clave_privada, clave_publica)
    def Sign(self, mensaje, clave_privada, clave_publica):
        return rsa_def.sign(mensaje, clave_privada, clave_publica)
    def CheckSign(self, mensaje, firma, clave_publica):
        return rsa_def.checksign(mensaje, firma, clave_publica)

class Colors():
    HEADER = '\033[95m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    ORANGE = '\033[93m'
    RED = '\033[91m'
    OK = 0
    INFO = 1
    WARNING = 2
    FAIL = 3
    ERROR = 4
    def __init__(self):
        import colors
        self.colors = colors
    def Reset(self):
        return self.colors.reset_color()
    def StatusText(self, style, text):
        return self.colors.status(style, text)
        
class SQL():
    def __init__(self, database):
        self.sqllite = cyfsql.CyfSQL(database)
    def Load(self):
        return self.sqllite.load()
    def Save(self):
        return self.sqllite.save()
    def Close(self):
        return self.sqllite.close()
    def Query(self, query):
        return self.sqllite.query(query)
    def FetchAll(self):
        return self.sqllite.fetchall()
            
__name__ = "CyfNet Library"
__version__ = "1.0"


import sqlite3 as dbapi
class CyfSQL():
    def __init__(self, filename):
        self.db = filename
    def load(self):
        self.bbdd = dbapi.connect(self.db)
        self.cursor = self.bbdd.cursor()
    def save(self):
        self.bbdd.commit()
    def close(self):
        self.cursor.close()
        self.bbdd.close()
    def query(self, query):
        self.cursor.execute(query)
    def fetchall(self):
        return self.cursor.fetchall()
    def createtable(self, name, values):
        q= "create table " + name + " ("
        for x in values:
            q = q + x + ","
        q = q[:len(q)-1]
        q = q + ")"
        return q


from distutils.core import setup
import cyflib
long_desc = open("cyflib/long_desc.txt").read()
setup(
    name='cyflib',
    version=cyflib.__version__,
    author='inobagger',
    author_email='inobagger@gmail.com',
    packages=['cyflib', 'cyflib/packages/ecdsa', 'cyflib/packages'],
    data_files=[('txt', ['cyflib/readme.txt', 'cyflib/license.txt', 'cyflib/changes.txt', 'cyflib/long_desc.txt'])],
    url='http://cyfnet.com/cyflib',
    license='Creative Commons Attribution 4.0 International Public License',
    description='CyfNet Python Library',
    download_url='http://cyfnet.com/cyflib/download',
    long_description=long_desc
)



HEADER = '\033[95m'
OKBLUE = '\033[94m'
OKGREEN = '\033[92m'
WARNING = '\033[93m'
FAIL = '\033[91m'
ENDC = '\033[0m'
def reset_color():
    return ENDC
def status(estilo, texto):
    if(estilo==0):
        return "[ " + OKGREEN + "OK" + ENDC + " ] " + texto
    elif(estilo==1):
        return "[" + OKBLUE + "INFO" + ENDC + "] " + texto
    elif(estilo==2):
        return "[" + WARNING + "WARN" + ENDC + "] " + texto
    elif(estilo==3):
        return "[" + FAIL + "FAIL" + ENDC + "] " + texto
    elif(estilo==4):
        return "[" + FAIL + "ERROR" + ENDC + "] " + texto
